==> julian-0.1.0.tar.gz: lib/src/julian.dart <==
// Copyright (c) 2016, Christian Stewart. All rights reserved. Use of this
// source code is governed by a BSD-style license that can be found in the
// LICENSE file.

import 'gregorian.dart';
import 'julian_base.dart';

/// A day in the proleptic Julian calendar.
class Julian implements CalendarDay {
  /// The Julian year, as in astronomical year numbering. So `year` can also be
  /// zero and negative.
  final int year;

  /// The month `1..12`.
  final int month;

  /// The day `1..31`.
  final int day;

  static const int daysIn28Years = 10227;

  /// The Julian day number is 0 on calendar day Julian(-4712, 1, 1).
  ///
  /// More commonly written as Julian 4713 BC January 1st.
  static const Julian julianDayNumberZeroDate = const Julian(-4712, 1, 1);

  static const num meanYearLengthInDays = 365.25;

  static const Duration meanYearLengthAsDuration =
      const Duration(days: 365, hours: 6);

  /// Creates a new calendar day in the proleptic Julian calendar.
  const Julian(this.year, this.month, this.day);

  /// Creates a new calendar day in the proleptic Julian calendar.
  ///
  /// `year` is interpreted as a BC format year and converted to astronomical
  /// year numbering.
  const Julian.fromBC(int year, this.month, this.day) : this.year = -(year - 1);

  factory Julian.fromGregorian(Gregorian gregorian) =>
      new Julian.fromJulianDayNumber(gregorian.julianDayNumber);

  factory Julian.fromJulianDayNumber(int jdn) {
    // Julian(0, 3, 1) <==> 1721118 julian day number
    const int j0 = 1721118;
    // There are 10227 days in 28 Julian years. The Julian calendar repeats
    // exactly after 28 years. Thus adding a multiple of 10227 days does not
    // cause the day/month values to change.
    final num daysDisplacement = (jdn >= j0)
        ? 0
        : ((jdn - j0).abs() / daysIn28Years).ceil() * daysIn28Years;
    // Days from the displaced date to j0
    int daysFromJ0 = jdn + daysDisplacement - j0,
        yearsFromJ0 = daysFromJ0 ~/ meanYearLengthInDays,
        y = yearsFromJ0 - (daysDisplacement ~/ meanYearLengthInDays);
    int daysFromMarch1 = daysFromJ0 - (yearsFromJ0 * 365 + yearsFromJ0 ~/ 4);
    // March <=> m = 0, April <=> m = 1, ...
    int m = (((daysFromMarch1 + 1 + 30.19694) / 30.6).floor() - 1);
    int d = daysFromMarch1 + 1 - (m * 30.6).round();
    m += 3;
    if (m > 12) {
      m -= 12;
      y += 1;
    }
    if (m == 2 && d > 28) {
      // Note: this d can be up to 367 on leap years!
      final int febLength = isLeapYear(y) ? 29 : 28;
      final int surplus = d - febLength;
      if (surplus > 0) {
        m = 3;
        d = surplus;
      }
    }
    return new Julian(y, m, d);
  }

  /// The corresponding Julian day number for `this`.
  int get julianDayNumber {
    // Julian day number is 0 for Julian(-4712, 1, 1) (beginning at 12 noon).
    // Julian calendar repeats exactly after 28 years.
    // 8400 is a multiple of 28 and also 8400 - 4712 >> 0.
    // Thus 8400 is a suitable multiple for a period of years to displace the
    // years of the epoch and this() to values > 0000/Mar/01 without
    // invalidating the month and day values.
    final int yearsDisplacement =
        8400 * (year >= 0 ? 1 : (1 + year.abs() / 8400).ceil());
    // Days from y/m/d (>= 0000/Mar/01) to 0000/Mar/01 (Julian)
    int daysFrom_0000_03_01(y, m, d) {
      // Convert to a march year start, so that leap days are an appendage
      if (m < 3) {
        y -= 1;
        m += 12;
      }
      return (y * 365) +
          (y ~/ 4) +
          // Gives the number of days since March 1 and comes from the
          // repetition of days in the month from March in groups of five:
          // Mar–Jul: 31 30 31 30 31
          // Aug–Dec: 31 30 31 30 31
          // Jan–Feb: 31 28
          (((m * 153 + 3) ~/ 5) - 92) +
          (d - 1);
    }
    return daysFrom_0000_03_01(year + yearsDisplacement, month, day) -
        daysFrom_0000_03_01(julianDayNumberZeroDate.year + yearsDisplacement,
            julianDayNumberZeroDate.month, julianDayNumberZeroDate.day);
  }

  /// Returns true if `year` is a Julian leap year.
  static bool isLeapYear(int year) => year.abs() % 4 == 0;

  /// Returns a Gregorian with the same Julian day number as `this`.
  Gregorian toGregorian() => new Gregorian.fromJulianDayNumber(julianDayNumber);

  /// Returns a list with three elements: [year], [month] and [day] in that order.
  List<int> toList() => [year, month, day];

  String toString() {
    return 'Julian(year: $year, month: $month, day: $day)';
  }
}

==> julian-0.1.1.tar.gz: lib/src/julian.dart <==
// Copyright (c) 2016, Christian Stewart. All rights reserved. Use of this
// source code is governed by a BSD-style license that can be found in the
// LICENSE file.

import 'gregorian.dart';
import 'julian_base.dart';

/// A day in the proleptic Julian calendar.
class Julian implements CalendarDay {
  /// The Julian [year], as in astronomical year numbering.
  ///
  /// [year] can be `0` and negative.
  final int year;

  /// The [month] `1..12`.
  final int month;

  /// The [day] `1..31`.
  final int day;

  static const int daysIn28Years = 10227;

  /// The Julian day number is `0` on calendar day Julian(-4712, 1, 1).
  ///
  /// More commonly written as Julian 4713 BC January 1st.
  static const Julian julianDayNumberZeroDate = const Julian(-4712, 1, 1);

  static const num meanYearLengthInDays = 365.25;

  static const Duration meanYearLengthAsDuration =
      const Duration(days: 365, hours: 6);

  /// Creates a new calendar day in the proleptic Julian calendar.
  const Julian(this.year, this.month, this.day);

  /// Creates a new calendar day in the proleptic Julian calendar.
  ///
  /// [year] is interpreted as a BC format year and converted to astronomical
  /// year numbering.
  const Julian.fromBC(int year, this.month, this.day) : this.year = -(year - 1);

  factory Julian.fromGregorian(Gregorian gregorian) =>
      new Julian.fromJulianDayNumber(gregorian.julianDayNumber);

  factory Julian.fromJulianDayNumber(int jdn) {
    // Julian(0, 3, 1) <==> 1721118 julian day number
    const int j0 = 1721118;
    // There are 10227 days in 28 Julian years. The Julian calendar repeats
    // exactly after 28 years. Thus adding a multiple of 10227 days does not
    // cause the day/month values to change.
    final num daysDisplacement = (jdn >= j0)
        ? 0
        : ((jdn - j0).abs() / daysIn28Years).ceil() * daysIn28Years;
    // Days from the displaced date to j0
    int daysFromJ0 = jdn + daysDisplacement - j0,
        yearsFromJ0 = daysFromJ0 ~/ meanYearLengthInDays,
        y = yearsFromJ0 - (daysDisplacement ~/ meanYearLengthInDays);
    int daysFromMarch1 = daysFromJ0 - (yearsFromJ0 * 365 + yearsFromJ0 ~/ 4);
    // March <=> m = 0, April <=> m = 1, ...
    int m = (((daysFromMarch1 + 1 + 30.19694) / 30.6).floor() - 1);
    int d = daysFromMarch1 + 1 - (m * 30.6).round();
    m += 3;
    if (m > 12) {
      m -= 12;
      y += 1;
    }
    if (m == 2 && d > 28) {
      // Note: this d can be up to 367 on leap years!
      final int febLength = isLeapYear(y) ? 29 : 28;
      final int surplus = d - febLength;
      if (surplus > 0) {
        m = 3;
        d = surplus;
      }
    }
    return new Julian(y, m, d);
  }

  /// The Julian day number for [this].
  int get julianDayNumber {
    // Julian day number is 0 for Julian(-4712, 1, 1) (beginning at 12 noon).
    // Julian calendar repeats exactly after 28 years.
    // 8400 is a multiple of 28 and also 8400 - 4712 >> 0.
    // Thus 8400 is a suitable multiple for a period of years to displace the
    // years of the epoch and this() to values > 0000/Mar/01 without
    // invalidating the month and day values.
    final int yearsDisplacement =
        8400 * (year >= 0 ? 1 : (1 + year.abs() / 8400).ceil());
    // Days from y/m/d (>= 0000/Mar/01) to 0000/Mar/01 (Julian)
    int daysFrom_0000_03_01(y, m, d) {
      // Convert to a march year start, so that leap days are an appendage
      if (m < 3) {
        y -= 1;
        m += 12;
      }
      return (y * 365) +
          (y ~/ 4) +
          // Gives the number of days since March 1 and comes from the
          // repetition of days in the month from March in groups of five:
          // Mar–Jul: 31 30 31 30 31
          // Aug–Dec: 31 30 31 30 31
          // Jan–Feb: 31 28
          (((m * 153 + 3) ~/ 5) - 92) +
          (d - 1);
    }
    return daysFrom_0000_03_01(year + yearsDisplacement, month, day) -
        daysFrom_0000_03_01(julianDayNumberZeroDate.year + yearsDisplacement,
            julianDayNumberZeroDate.month, julianDayNumberZeroDate.day);
  }

  /// Returns `true` if [year] is a Julian leap year.
  static bool isLeapYear(int year) => year.abs() % 4 == 0;

  /// Returns a [Gregorian] with the same Julian day number as [this].
  Gregorian toGregorian() => new Gregorian.fromJulianDayNumber(julianDayNumber);

  /// Returns a list with three elements: [year], [month] and [day] in that
  /// order.
  List<int> toList() => [year, month, day];

  String toString() {
    return 'Julian(year: $year, month: $month, day: $day)';
  }
}
